@@ -691,6 +691,7 @@ wait_and_handle_event (long waittime, CameraEventType *type, int download) {
 	evtype = GP_EVENT_UNKNOWN;
 	result = gp_camera_wait_for_event(gp_params.camera, waittime, type, &data, gp_params.context);
 	if (result == GP_ERROR_NOT_SUPPORTED) {
+		*type = GP_EVENT_TIMEOUT;
 		usleep(waittime*1000);
 		return GP_OK;
 	}
@@ -786,7 +787,7 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 			gettimeofday (&expose_end_time, NULL);
 			expose_end_time.tv_sec += glob_bulblength;
 			waittime = timediff_now (&expose_end_time);
-			while(waittime > 0) {
+			while (waittime > 0) {
 				result = wait_and_handle_event(waittime, &evtype, download);
 				if (result != GP_OK)
 					return result;
@@ -873,8 +874,13 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 					waittime = timediff_now (&next_pic_time);
 				} while (waittime > 0);
 			} else {
-				/* even though we have no time, drain the queue */
-				result = wait_and_handle_event (1, NULL, download);
+				/* even though we have no time, we need to drain the queue to capture downloads to-do */
+				evtype = GP_EVENT_UNKNOWN;
+				while (evtype != GP_EVENT_TIMEOUT) {
+					result = wait_and_handle_event (1, &evtype, download);
+					if (result != GP_OK)
+						break;
+				}
 				if (result != GP_OK)
 					break;
 				if (!(gp_params.flags & FLAGS_QUIET) && glob_interval)
@@ -1329,7 +1335,7 @@ cb_arg_run (poptContext __unused__ ctx,
 		params->p.r = capture_generic (GP_CAPTURE_IMAGE, arg, 1);
 		break;
 	case ARG_CAPTURE_MOVIE:
-		params->p.r = capture_generic (GP_CAPTURE_MOVIE, arg, 0);
+		params->p.r = action_camera_capture_movie (&gp_params, arg);
 		break;
 	case ARG_CAPTURE_PREVIEW:
 		params->p.r = action_camera_capture_preview (&gp_params);
@@ -1584,9 +1590,12 @@ report_failure (int result, int argc, char **argv)
 		fprintf (stderr, _("Operation cancelled.\n"));
 		return;
 	}
-
-	fprintf (stderr, _("*** Error (%i: '%s') ***       \n\n"), result,
-		gp_result_as_string (result));
+	if (result == -2000) {
+		fprintf (stderr, _("*** Error: No camera found. ***\n\n"));
+	} else {
+		fprintf (stderr, _("*** Error (%i: '%s') ***       \n\n"), result,
+			gp_result_as_string (result));
+	}
 	if (!debug_option_given) {
 		int n;
 		printf (_("For debugging messages, please use "
@@ -1755,8 +1764,8 @@ main (int argc, char **argv, char **envp)
 		 ARG_CAPTURE_IMAGE, N_("Capture an image"), NULL},
 		{"capture-image-and-download", '\0', POPT_ARG_NONE, NULL,
 		 ARG_CAPTURE_IMAGE_AND_DOWNLOAD, N_("Capture an image and download it"), NULL},
-		{"capture-movie", '\0', POPT_ARG_NONE, NULL,
-		 ARG_CAPTURE_MOVIE, N_("Capture a movie"), NULL},
+		{"capture-movie", '\0', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, NULL,
+		 ARG_CAPTURE_MOVIE, N_("Capture a movie"), N_("COUNT or SECONDS")},
 		{"capture-sound", '\0', POPT_ARG_NONE, NULL,
 		 ARG_CAPTURE_SOUND, N_("Capture an audio clip"), NULL},
 		{"capture-tethered", '\0', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, NULL,
@@ -2059,6 +2068,7 @@ main (int argc, char **argv, char **envp)
 			CR_MAIN (action_camera_set_port (&gp_params, path));
 
                 } else if (!count) {
+			int ret;
 			/*
 			 * No camera detected. Have a look at the settings.
 			 * Ignore errors here, it might be a serial one.
@@ -2067,7 +2077,12 @@ main (int argc, char **argv, char **envp)
 				action_camera_set_model (&gp_params, buf);
 			if (gp_setting_get ("gphoto2", "port", buf) >= 0)
 				action_camera_set_port (&gp_params, buf);
-
+			ret = gp_camera_init (gp_params.camera, gp_params.context);
+			if (ret != GP_OK) {
+				if (ret == GP_ERROR_BAD_PARAMETERS)
+					ret = -2000;
+				CR_MAIN (ret);
+			}
 		} else {
 			/* If --port override, search the model with the same port. */
 			if (info.type != GP_PORT_NONE) {
